/****************************************************************************
** Meta object code from reading C++ file 'dbushotzone.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/global_util/dbus/dbushotzone.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbushotzone.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11DBusHotzoneE_t {};
} // unnamed namespace

template <> constexpr inline auto DBusHotzone::qt_create_metaobjectdata<qt_meta_tag_ZN11DBusHotzoneE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DBusHotzone",
        "__propertyChanged__",
        "",
        "QDBusMessage",
        "msg",
        "BottomLeftAction",
        "QDBusPendingReply<QString>",
        "BottomRightAction",
        "EnableZoneDetected",
        "QDBusPendingReply<>",
        "in0",
        "SetBottomLeft",
        "SetBottomRight",
        "SetTopLeft",
        "SetTopRight",
        "TopLeftAction",
        "TopRightAction"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot '__propertyChanged__'
        QtMocHelpers::SlotData<void(const QDBusMessage &)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'BottomLeftAction'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(5, 2, QMC::AccessPublic, 0x80000000 | 6),
        // Slot 'BottomRightAction'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(7, 2, QMC::AccessPublic, 0x80000000 | 6),
        // Slot 'EnableZoneDetected'
        QtMocHelpers::SlotData<QDBusPendingReply<>(bool)>(8, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'SetBottomLeft'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(11, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'SetBottomRight'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(12, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'SetTopLeft'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(13, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'SetTopRight'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(14, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'TopLeftAction'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(15, 2, QMC::AccessPublic, 0x80000000 | 6),
        // Slot 'TopRightAction'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(16, 2, QMC::AccessPublic, 0x80000000 | 6),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DBusHotzone, qt_meta_tag_ZN11DBusHotzoneE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DBusHotzone::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11DBusHotzoneE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11DBusHotzoneE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11DBusHotzoneE_t>.metaTypes,
    nullptr
} };

void DBusHotzone::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DBusHotzone *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->__propertyChanged__((*reinterpret_cast<std::add_pointer_t<QDBusMessage>>(_a[1]))); break;
        case 1: { QDBusPendingReply<QString> _r = _t->BottomLeftAction();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<QString> _r = _t->BottomRightAction();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<> _r = _t->EnableZoneDetected((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<> _r = _t->SetBottomLeft((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<> _r = _t->SetBottomRight((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<> _r = _t->SetTopLeft((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<> _r = _t->SetTopRight((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<QString> _r = _t->TopLeftAction();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<QString> _r = _t->TopRightAction();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusMessage >(); break;
            }
            break;
        }
    }
}

const QMetaObject *DBusHotzone::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DBusHotzone::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11DBusHotzoneE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int DBusHotzone::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
