/****************************************************************************
** Meta object code from reading C++ file 'dbuskeyboard.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/global_util/dbus/dbuskeyboard.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbuskeyboard.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12DBusKeyboardE_t {};
} // unnamed namespace

template <> constexpr inline auto DBusKeyboard::qt_create_metaobjectdata<qt_meta_tag_ZN12DBusKeyboardE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DBusKeyboard",
        "CapslockToggleChanged",
        "",
        "CurrentLayoutChanged",
        "CursorBlinkChanged",
        "RepeatDelayChanged",
        "RepeatEnabledChanged",
        "RepeatIntervalChanged",
        "UserLayoutListChanged",
        "UserOptionListChanged",
        "__propertyChanged__",
        "QDBusMessage",
        "msg",
        "AddLayoutOption",
        "QDBusPendingReply<>",
        "in0",
        "AddUserLayout",
        "ClearLayoutOption",
        "DeleteLayoutOption",
        "DeleteUserLayout",
        "GetLayoutDesc",
        "QDBusPendingReply<QString>",
        "LayoutList",
        "QDBusPendingReply<KeyboardLayoutList>",
        "Reset",
        "CapslockToggle",
        "CurrentLayout",
        "CursorBlink",
        "RepeatDelay",
        "RepeatEnabled",
        "RepeatInterval",
        "UserLayoutList",
        "UserOptionList"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'CapslockToggleChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'CurrentLayoutChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'CursorBlinkChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'RepeatDelayChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'RepeatEnabledChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'RepeatIntervalChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'UserLayoutListChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'UserOptionListChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot '__propertyChanged__'
        QtMocHelpers::SlotData<void(const QDBusMessage &)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'AddLayoutOption'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(13, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'AddUserLayout'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(16, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'ClearLayoutOption'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(17, 2, QMC::AccessPublic, 0x80000000 | 14),
        // Slot 'DeleteLayoutOption'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(18, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'DeleteUserLayout'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(19, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'GetLayoutDesc'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(const QString &)>(20, 2, QMC::AccessPublic, 0x80000000 | 21, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'LayoutList'
        QtMocHelpers::SlotData<QDBusPendingReply<KeyboardLayoutList>()>(22, 2, QMC::AccessPublic, 0x80000000 | 23),
        // Slot 'Reset'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(24, 2, QMC::AccessPublic, 0x80000000 | 14),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'CapslockToggle'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'CurrentLayout'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'CursorBlink'
        QtMocHelpers::PropertyData<int>(27, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'RepeatDelay'
        QtMocHelpers::PropertyData<uint>(28, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'RepeatEnabled'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'RepeatInterval'
        QtMocHelpers::PropertyData<uint>(30, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'UserLayoutList'
        QtMocHelpers::PropertyData<QStringList>(31, QMetaType::QStringList, QMC::DefaultPropertyFlags, 6),
        // property 'UserOptionList'
        QtMocHelpers::PropertyData<QStringList>(32, QMetaType::QStringList, QMC::DefaultPropertyFlags, 7),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DBusKeyboard, qt_meta_tag_ZN12DBusKeyboardE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DBusKeyboard::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DBusKeyboardE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DBusKeyboardE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12DBusKeyboardE_t>.metaTypes,
    nullptr
} };

void DBusKeyboard::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DBusKeyboard *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->CapslockToggleChanged(); break;
        case 1: _t->CurrentLayoutChanged(); break;
        case 2: _t->CursorBlinkChanged(); break;
        case 3: _t->RepeatDelayChanged(); break;
        case 4: _t->RepeatEnabledChanged(); break;
        case 5: _t->RepeatIntervalChanged(); break;
        case 6: _t->UserLayoutListChanged(); break;
        case 7: _t->UserOptionListChanged(); break;
        case 8: _t->__propertyChanged__((*reinterpret_cast<std::add_pointer_t<QDBusMessage>>(_a[1]))); break;
        case 9: { QDBusPendingReply<> _r = _t->AddLayoutOption((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<> _r = _t->AddUserLayout((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<> _r = _t->ClearLayoutOption();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 12: { QDBusPendingReply<> _r = _t->DeleteLayoutOption((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 13: { QDBusPendingReply<> _r = _t->DeleteUserLayout((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 14: { QDBusPendingReply<QString> _r = _t->GetLayoutDesc((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 15: { QDBusPendingReply<KeyboardLayoutList> _r = _t->LayoutList();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<KeyboardLayoutList>*>(_a[0]) = std::move(_r); }  break;
        case 16: { QDBusPendingReply<> _r = _t->Reset();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusMessage >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DBusKeyboard::*)()>(_a, &DBusKeyboard::CapslockToggleChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusKeyboard::*)()>(_a, &DBusKeyboard::CurrentLayoutChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusKeyboard::*)()>(_a, &DBusKeyboard::CursorBlinkChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusKeyboard::*)()>(_a, &DBusKeyboard::RepeatDelayChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusKeyboard::*)()>(_a, &DBusKeyboard::RepeatEnabledChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusKeyboard::*)()>(_a, &DBusKeyboard::RepeatIntervalChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusKeyboard::*)()>(_a, &DBusKeyboard::UserLayoutListChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DBusKeyboard::*)()>(_a, &DBusKeyboard::UserOptionListChanged, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->capslockToggle(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->currentLayout(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->cursorBlink(); break;
        case 3: *reinterpret_cast<uint*>(_v) = _t->repeatDelay(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->repeatEnabled(); break;
        case 5: *reinterpret_cast<uint*>(_v) = _t->repeatInterval(); break;
        case 6: *reinterpret_cast<QStringList*>(_v) = _t->userLayoutList(); break;
        case 7: *reinterpret_cast<QStringList*>(_v) = _t->userOptionList(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCapslockToggle(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setCurrentLayout(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setCursorBlink(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setRepeatDelay(*reinterpret_cast<uint*>(_v)); break;
        case 4: _t->setRepeatEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setRepeatInterval(*reinterpret_cast<uint*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *DBusKeyboard::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DBusKeyboard::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DBusKeyboardE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int DBusKeyboard::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void DBusKeyboard::CapslockToggleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DBusKeyboard::CurrentLayoutChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void DBusKeyboard::CursorBlinkChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void DBusKeyboard::RepeatDelayChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void DBusKeyboard::RepeatEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void DBusKeyboard::RepeatIntervalChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void DBusKeyboard::UserLayoutListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void DBusKeyboard::UserOptionListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
